@@ -1,5 +1,7 @@
 
-__version__ = '0.5'
+## for now, this needs to be changed in setup.py also until I find a
+## better solution
+__version__ = '0.6'
 __author__ = 'meejah'
 __contact__ = 'meejah@meejah.ca'
 __url__ = 'https://github.com/meejah/txtorcon'
@@ -12,8 +14,10 @@ from txtorcon.stream import Stream
 from txtorcon.torcontrolprotocol import TorControlProtocol, TorProtocolError, TorProtocolFactory, DEFAULT_VALUE
 from txtorcon.torstate import TorState, build_tor_connection
 from txtorcon.torconfig import TorConfig, HiddenService, TorProcessProtocol, TCPHiddenServiceEndpoint, launch_tor
+from txtorcon.torinfo import TorInfo
 from txtorcon.addrmap import AddrMap
 from txtorcon.addrmap import Addr
+from txtorcon.torinfo import TorInfo
 import util
 import interface
 from txtorcon.interface import *
@@ -23,8 +27,10 @@ __all__ = ["Router",
            "Stream",
            "TorControlProtocol", "TorProtocolError",
            "TorState", "DEFAULT_VALUE",
+           "TorInfo",
            "build_tor_connection", "launch_tor",
            "TorConfig", "HiddenService", "TorProcessProtocol",
+           "TorInfo",
            "TCPHiddenServiceEndpoint",
            "AddrMap",
            "util", "interface",
@@ -34,4 +40,3 @@ __all__ = ["Router",
            "ICircuitContainer", "ICircuitListener", "CircuitListenerMixin",
            "IRouterContainer", "IAddrListener"
            ]
-
