@@ -16,7 +16,8 @@ from txtorcon.torcontrolprotocol import parse_keywords, DEFAULT_VALUE, TorProtoc
 from txtorcon.util import delete_file_or_tree, find_keywords
 from txtorcon.log import txtorlog
 
-from interface import ITorControlProtocol, IRouterContainer, ICircuitListener, ICircuitContainer, IStreamListener, IStreamAttacher
+from txtorcon.interface import ITorControlProtocol, IRouterContainer, ICircuitListener
+from txtorcon.interface import ICircuitContainer, IStreamListener, IStreamAttacher
 from spaghetti import FSM, State, Transition
 
 import os
@@ -91,7 +92,7 @@ class TCPHiddenServiceEndpoint(object):
         self.data_dir = data_dir
         self.onion_uri = None
         self.onion_private_key = None
-        if self.data_dir != None:
+        if self.data_dir is not None:
             self._update_onion()
 
         else:
@@ -114,8 +115,8 @@ class TCPHiddenServiceEndpoint(object):
         `onion_private_key` members.
         """
 
-        hn = os.path.join(self.hiddenservice.dir,'hostname')
-        pk = os.path.join(self.hiddenservice.dir,'private_key')
+        hn = os.path.join(self.hiddenservice.dir, 'hostname')
+        pk = os.path.join(self.hiddenservice.dir, 'private_key')
         try:
             with open(hn, 'r') as hnfile:
                 self.onion_uri = hnfile.read().strip()
@@ -140,7 +141,8 @@ class TCPHiddenServiceEndpoint(object):
         self.listen_port = 80
 
         self.hiddenservice = HiddenService(self.config, self.data_dir,
-                                           ['%d 127.0.0.1:%d' % (self.public_port, self.listen_port)])
+                                           ['%d 127.0.0.1:%d' % (self.public_port,
+                                                                 self.listen_port)])
         self.config.HiddenServices.append(self.hiddenservice)
         return arg
 
@@ -149,7 +151,7 @@ class TCPHiddenServiceEndpoint(object):
         handle errors. FIXME
         """
 
-        print "ERROR",f
+        print "ERROR", f
         return f
 
     def listen(self, protocolfactory):
@@ -193,7 +195,8 @@ class TCPHiddenServiceEndpoint(object):
             return failure
         self.listen_port = self.port_generator()
         ## we do want to overwrite the whole list, not append
-        self.hiddenservice.ports = ['%d 127.0.0.1:%d' % (self.public_port, self.listen_port)]
+        self.hiddenservice.ports = ['%d 127.0.0.1:%d' % (self.public_port,
+                                                         self.listen_port)]
         d = self.config.save()
         d.addCallback(self._create_listener).addErrback(self._retry_local_port)
         return d
@@ -218,21 +221,24 @@ class TCPHiddenServiceEndpoint(object):
         port.onion_port = self.public_port
         return port
 
+
 class TorProcessProtocol(protocol.ProcessProtocol):
 
-    def __init__(self, connection_creator, progress_updates = None):
+    def __init__(self, connection_creator, progress_updates=None):
         """
         This will read the output from a Tor process and attempt a
         connection to its control port when it sees any 'Bootstrapped'
         message on stdout. You probably don't need to use this
-        directly except as the return value from the :func:`txtorcon.launch_tor`
-        method. tor_protocol contains a valid :class:`txtorcon.TorControlProtocol`
-        instance by that point.
+        directly except as the return value from the
+        :func:`txtorcon.launch_tor` method. tor_protocol contains a
+        valid :class:`txtorcon.TorControlProtocol` instance by that
+        point.
 
         connection_creator is a callable that should return a Deferred
-        that callbacks with a :class:`txtorcon.TorControlProtocol`; see :func:`txtorcon.launch_tor` for
-        the default one which is a functools.partial that will call
-        .connect(TorProtocolFactory()) on an appropriate
+        that callbacks with a :class:`txtorcon.TorControlProtocol`;
+        see :func:`txtorcon.launch_tor` for the default one which is a
+        functools.partial that will call
+        ``connect(TorProtocolFactory())`` on an appropriate
         :api:`twisted.internet.endpoints.TCP4ClientEndpoint`
 
         :param connection_creator: A no-parameter callable which
@@ -242,7 +248,7 @@ class TorProcessProtocol(protocol.ProcessProtocol):
             updates with three args: percent, tag, summary
 
         :ivar tor_protocol: The TorControlProtocol instance connected
-            to the Tor this :api:`twisted.internet.protocol.ProcessProtocol <ProcessProtocol>` is speaking to. Will be valid
+            to the Tor this :api:`twisted.internet.protocol.ProcessProtocol <ProcessProtocol>`` is speaking to. Will be valid
             when the `connected_cb` callback runs.
 
         :ivar connected_cb: Triggered when the Tor process we
@@ -363,10 +369,8 @@ class TorProcessProtocol(protocol.ProcessProtocol):
 
 
 def launch_tor(config, reactor,
-               control_port=9052,
-               data_directory=None,
                tor_binary='/usr/sbin/tor',
-               progress_updates = None,
+               progress_updates=None,
                connection_creator=None):
     """
     launches a new Tor process with the given config.
@@ -374,18 +378,16 @@ def launch_tor(config, reactor,
     If Tor prints anything on stderr, we kill off the process, close
     the TorControlProtocol and raise an exception.
 
-    :param config: an instance of :class:`txtorcon.TorConfig` with any configuration
-        values you want. :meth:`txtorcon.TorConfig.save` should have been
-        called already (anything unsaved won't make it into the torrc
-        produced). Note that the control_port and data_directory
-        parameters to this method override anything in the config.
+    :param config: an instance of :class:`txtorcon.TorConfig` with any
+        configuration values you want. :meth:`txtorcon.TorConfig.save`
+        should have been called already (anything unsaved won't make
+        it into the torrc produced). If ControlPort isn't set, 9052 is
+        used; if DataDirectory isn't set, tempdir is used to create
+        one.
 
     :param reactor: a Twisted IReactorCore implementation (usually
         twisted.internet.reactor)
 
-    :param control_port: which port the launched Tor process will
-        listen on; anything set in config is overridden.
-
     :param tor_binary: path to the Tor binary to run.
 
     :param progress_updates: a callback which gets progress updates; gets as
@@ -393,9 +395,9 @@ def launch_tor(config, reactor,
 
     :param connection_creator: is mostly available to ease testing, so
         you probably don't want to supply this. If supplied, it is a
-        callable that should return a Deferred that delivers an :api:`twisted.internet.interfaces.IProtocol <IProtocol>`
-        or ConnectError. See
-        :api:`twisted.internet.interfaces.IStreamClientEndpoint`.connect
+        callable that should return a Deferred that delivers an
+        :api:`twisted.internet.interfaces.IProtocol <IProtocol>` or ConnectError.
+        See :api:`twisted.internet.interfaces.IStreamClientEndpoint`.connect
 
     :return: a Deferred which callbacks with a TorProcessProtocol
         connected to the fully-bootstrapped Tor; this has a
@@ -425,17 +427,29 @@ def launch_tor(config, reactor,
     ## config and get Tor to load that...which might be the best
     ## option anyway.
 
-    if data_directory is None:
+    if config.needs_save():
+        log.msg("Config was unsaved when launch_tor() called; calling save().")
+        config.save()
+
+    try:
+        data_directory = config.DataDirectory
+        user_set_data_directory = True
+    except KeyError:
+        user_set_data_directory = False
         data_directory = tempfile.mkdtemp(prefix='tortmp')
+        config.DataDirectory = data_directory
+
+    try:
+        control_port = config.ControlPort
+    except KeyError:
+        control_port = 9052
+        config.ControlPort = control_port
+
+    config.CookieAuthentication = 1
+    config.__OwningControllerProcess = os.getpid()
+    config.save()
 
     (fd, torrc) = tempfile.mkstemp(prefix='tortmp')
-
-    config.DataDirectory = data_directory
-    config.ControlPort = control_port
-    config.CookieAuthentication = 1
-    config.SocksPort = 0
-    config.__OwningControllerProcess = os.getpid()
-
     os.write(fd, config.create_torrc())
     os.close(fd)
 
@@ -446,13 +460,25 @@ def launch_tor(config, reactor,
                                                TorProtocolFactory())
     process_protocol = TorProcessProtocol(connection_creator, progress_updates)
 
-    # we do both because this process might be shut down way before
-    # the reactor, but if the reactor bombs out without the subprocess
-    # getting closed cleanly, we'll want the system shutdown events
-    # triggered
-    process_protocol.to_delete = [torrc, data_directory]
-    reactor.addSystemEventTrigger('before', 'shutdown',
-                                  functools.partial(delete_file_or_tree, torrc, data_directory))
+    # we set both to_delete and the shutdown events because this
+    # process might be shut down way before the reactor, but if the
+    # reactor bombs out without the subprocess getting closed cleanly,
+    # we'll want the system shutdown events triggered so the temporary
+    # files get cleaned up
+
+    # we don't want to delete the user's directories, just our
+    # temporary ones
+    if user_set_data_directory:
+        process_protocol.to_delete = [torrc]
+        reactor.addSystemEventTrigger('before', 'shutdown',
+                                      functools.partial(delete_file_or_tree,
+                                                        torrc))
+    else:
+        process_protocol.to_delete = [torrc, data_directory]
+        reactor.addSystemEventTrigger('before', 'shutdown',
+                                      functools.partial(delete_file_or_tree,
+                                                        torrc,
+                                                        data_directory))
 
     try:
         transport = reactor.spawnProcess(process_protocol, tor_binary,
@@ -468,7 +494,6 @@ def launch_tor(config, reactor,
     return process_protocol.connected_cb
 
 
-
 class TorConfigType(object):
     """
     Base class for all configuration types, which function as parsers
@@ -489,51 +514,83 @@ class TorConfigType(object):
         """
         return s
 
+
 class Boolean(TorConfigType):
     def parse(self, s):
         if int(s):
             return True
         return False
 
+
+class Boolean_Auto(TorConfigType):
+    """
+    weird class-name, but see the parser for these which is *mostly*
+    just the classname <==> string from Tor, except for something
+    called Boolean+Auto which is replace()d to be Boolean_Auto
+    """
+
+    def parse(self, s):
+        if s == 'auto' or int(s) < 0:
+            return -1
+        if int(s):
+            return 1
+        return 0
+
+
 class Integer(TorConfigType):
     def parse(self, s):
         return int(s)
 
+
+class SignedInteger(Integer):
+    pass
+
+
 class Port(Integer):
     pass
 
+
 class TimeInterval(Integer):
     pass
 
+
 ## not actually used?
 class TimeMsecInterval(TorConfigType):
     pass
 
+
 class DataSize(Integer):
     pass
 
+
 class Float(TorConfigType):
     def parse(self, s):
         return float(s)
 
+
 ## unused also?
 class Time(TorConfigType):
     pass
 
+
 class CommaList(TorConfigType):
     def parse(self, s):
         return map(string.strip, s.split(','))
 
+
 ## FIXME: is this really a comma-list?
 class RouterList(CommaList):
     pass
 
+
 class String(TorConfigType):
     pass
 
+
 class Filename(String):
     pass
 
+
 class LineList(TorConfigType):
     def parse(self, s):
         if isinstance(s, types.ListType):
@@ -545,7 +602,11 @@ class LineList(TorConfigType):
             raise ValueError("Not valid for %s: %s" % (self.__class__, obj))
         return _ListWrapper(obj, functools.partial(instance.mark_unsaved, name))
 
-config_types = [Boolean, LineList, Integer, Port, TimeInterval, TimeMsecInterval, DataSize, Float, Time, CommaList, String, LineList, Filename, RouterList]
+config_types = [Boolean, Boolean_Auto, LineList, Integer, SignedInteger, Port,
+                TimeInterval, TimeMsecInterval,
+                DataSize, Float, Time, CommaList, String, LineList, Filename,
+                RouterList]
+
 
 def _wrapture(orig):
     """
@@ -562,6 +623,7 @@ def _wrapture(orig):
         return orig(*args)
     return foo
 
+
 class _ListWrapper(list):
     """
     Do some voodoo to wrap lists so that if you do anything to modify
@@ -586,6 +648,7 @@ class _ListWrapper(list):
     def __repr__(self):
         return '_ListWrapper' + super(_ListWrapper, self).__repr__()
 
+
 class HiddenService(object):
     """
     Because hidden service configuration is handled specially by Tor,
@@ -628,7 +691,8 @@ class HiddenService(object):
 
         if not isinstance(ports, types.ListType):
             ports = [ports]
-        self.ports = _ListWrapper(ports, functools.partial(self.conf.mark_unsaved, 'HiddenServices'))
+        self.ports = _ListWrapper(ports, functools.partial(self.conf.mark_unsaved,
+                                                           'HiddenServices'))
 
     def __setattr__(self, name, value):
         """
@@ -640,7 +704,8 @@ class HiddenService(object):
         if name in ['dir', 'version', 'authorize_client', 'ports'] and self.conf:
             self.conf.mark_unsaved('HiddenServices')
         if isinstance(value, types.ListType):
-            value = _ListWrapper(value, functools.partial(self.conf.mark_unsaved, 'HiddenServices'))
+            value = _ListWrapper(value, functools.partial(self.conf.mark_unsaved,
+                                                          'HiddenServices'))
         self.__dict__[name] = value
 
     def __getattr__(self, name):
@@ -663,12 +728,13 @@ class HiddenService(object):
             rtn.append(('HiddenServiceAuthorizeClient', self.authorize_client))
         return rtn
 
+
 class TorConfig(object):
     """
     This class abstracts out Tor's config so that you don't have to
     realize things like: in order to successfully set multiple listen
     addresses, you must put them all (and the or-ports) in one SETCONF
-    call. (FIXME: this isn't true yet)
+    call.
 
     Also, it gives easy access to all the configuration options
     present. This is done with lazy caching: the first time you access
@@ -681,6 +747,9 @@ class TorConfig(object):
     config items they will reflect the current state of Tor
     (i.e. *not* what's been set since the last save())
 
+    Note that you do not need to call save() if you're just using
+    TorConfig to create a .torrc file or for input to launch_tor().
+
     FIXME: It also listens on the CONF_CHANGED event to update the
     cached data in the event other controllers (etc) changed it. (Only
     exists in Git versions?)
@@ -738,9 +807,9 @@ class TorConfig(object):
         all over the place.
         """
 
-        if self.__dict__.has_key('_setup_'):
+        if '_setup_' in self.__dict__:
             name = self._find_real_name(name)
-            if not self.__dict__.has_key('_slutty_') and name.lower() != 'hiddenservices':
+            if '_slutty_' not in self.__dict__ and name.lower() != 'hiddenservices':
                 value = self.parsers[name].validate(value, self, name)
             if isinstance(value, types.ListType):
                 value = _ListWrapper(value, functools.partial(self.mark_unsaved, name))
@@ -751,7 +820,6 @@ class TorConfig(object):
         else:
             super(TorConfig, self).__setattr__(name, value)
 
-
     def __getattr__(self, name):
         """
         on purpose, we don't return self.saved if the key is in there
@@ -776,8 +844,37 @@ class TorConfig(object):
             return HiddenService
         return type(self.parsers[name])
 
+    def _conf_changed(self, arg):
+        """
+        internal callback. from control-spec:
+
+        4.1.18. Configuration changed
+
+          The syntax is:
+             StartReplyLine *(MidReplyLine) EndReplyLine
+
+             StartReplyLine = "650-CONF_CHANGED" CRLF
+             MidReplyLine = "650-" KEYWORD ["=" VALUE] CRLF
+             EndReplyLine = "650 OK"
+
+          Tor configuration options have changed (such as via a SETCONF or RELOAD
+          signal). KEYWORD and VALUE specify the configuration option that was changed.
+          Undefined configuration options contain only the KEYWORD.
+        """
+
+        conf = parse_keywords(arg, multiline_values=False)
+        for (k, v) in conf.items():
+            ## v will be txtorcon.DEFAULT_VALUE already from
+            ## parse_keywords if it was unspecified
+            self.config[self._find_real_name(k)] = v
+
     def bootstrap(self, *args):
-##        self.protocol.add_event_listener('CONF_CHANGED', self._conf_changed)
+        try:
+            self.protocol.add_event_listener('CONF_CHANGED', self._conf_changed)
+        except (RuntimeError, e):
+            ## for Tor versions which don't understand CONF_CHANGED
+            ## there's nothing we can really do.
+            log.msg("Can't listen for CONF_CHANGED event; won't stay up-to-date with other clients.")
         return self.protocol.get_info_raw("config/names").addCallbacks(self._do_setup, log.err).addCallback(self.do_post_bootstrap).addErrback(log.err)
 
     def do_post_bootstrap(self, *args):
@@ -789,7 +886,7 @@ class TorConfig(object):
 
     def mark_unsaved(self, name):
         name = self._find_real_name(name)
-        if self.config.has_key(name) and not self.unsaved.has_key(name):
+        if name in self.config and name not in self.unsaved:
             self.unsaved[name] = self.config[self._find_real_name(name)]
 
     def save(self):
@@ -843,9 +940,11 @@ class TorConfig(object):
             return d
 
         else:
+            self._save_completed()
             return defer.succeed(self)
 
-    def _save_completed(self, foo):
+    def _save_completed(self, *args):
+        '''internal callback'''
         self.__dict__['unsaved'] = {}
         return self
 
@@ -871,6 +970,11 @@ class TorConfig(object):
             if value == 'Dependant':
                 continue
 
+            ## there's a thing called "Boolean+Auto" which is -1 for
+            ## auto, 0 for false and 1 for true. could be nicer if it
+            ## was called AutoBoolean or something, but...
+            value = value.replace('+', '_')
+            
             inst = None
             # FIXME: put parser classes in dict instead?
             for cls in config_types:
@@ -908,7 +1012,7 @@ class TorConfig(object):
                 continue
             k, v = line.split('=')
             if k == 'HiddenServiceDir':
-                if directory != None:
+                if directory is not None:
                     hs.append(HiddenService(self, directory, ports, auth, ver))
                 directory = v
                 ports = []
